source('DesignFunctions.R')


############################  
## continuous endpoints  ###
############################

## Input:
## x: covaraites, n*p matrix. 
## a: treatments; takes values in {0, 1}
## y: observed efficacy outcomes
## r: observed risk outcomes
## nu0: threshold for proportional constraint  
## testx: test data for determining the enrollment status 

## Output:
## deltax: the estimated value for the decision function in training data   
## testdeltax: the estimated value for the decision function in testing data   
## hatf: the estimated decision rule on in training data   
## testhatf: the estimated decision rule on in testing data   
## cutoff: cutoff value for the decision function

run.continuous.example = function(x, a, y, r, nu0, testx){
  
  ## calculate \Delta(x)
  
  deltax     = calculate_deltax(x, a, y, link = 'con', newx = x)$deltax
  deltarx     = calculate_deltax(x, a, r, link = 'con', newx = x)$deltax
  
  ## calculate effect size for different cutoffs
  betacs = sigmas = nu_hatfcut = prop_hatfcut = rep(0,n)
  sortdeltax = sort(deltax)
  for (j in 1:length(deltax)){
      cut          = sortdeltax[j]
      hatfcut      = (deltax >= cut)
      nu_hatfcut[j]= mean(deltarx*(hatfcut>0))
      prop_hatfcut[j]= mean(hatfcut)
      #betacs[j]    = mean(y[hatfcut == 1 & a == 1]) - mean(y[hatfcut == 1 & a == 0])
      betacs[j]    = mean(deltax[hatfcut == 1])
      sigmas[j]    = sqrt(var(y[hatfcut == 1]))
  }
  
  ## proposed design
  cal_hatf   = calculate_constraint_hatf(deltax, deltarx, nu0 = nu0)
  
  hatf       = cal_hatf$hatf
  optlambda  = cal_hatf$lambda
  
  ## calculate decision functions for testdata
  testdeltax = calculate_deltax(x, a, y, link = 'con', newx = testx)$deltax
  testdeltarx = calculate_deltax(x, a, r, link = 'con', newx = testx)$deltax
  
  testhatf   = (testdeltax - optlambda*testdeltarx > 0)
  
  return(list(deltax = deltax, lambda = optlambda, hatf = hatf,
              testdeltax = testdeltax, testhatf = testhatf)) 
}

## generate data (example 1)
conlink  = function(x,a){
    n = length(a)
    #return(0.5+0.5*x - 0.1*a + .4*a*x)
    return((1 - 2*x[,1] +   x[,2] +  (1 - x[,1] - x[,2])*a/3))
}
conlinkrisk  = function(x,a){
    n = length(a)
    return((2 + x[,1] - 2 * x[,2] + (1 + x[,1] - x[,2])*a))
}


n      =  500; 
p      =  2;
x      =  matrix(rnorm(2*n),ncol=p)
a      =  rbinom(n,1,0.5)
y      =  rnorm(n,conlink(x,a),1)
r      =  rnorm(n,conlinkrisk(x,a),1)

testn  =  10000;
testx  =  matrix(rnorm(testn),ncol=p)

nu0    =  0.4;
con.result = run.continuous.example(x, a, y, r, nu0 = nu0, testx = testx)
testdeltax = con.result$testdeltax 
lambda     = con.result$lambda

############################  
##   survival endpoint  ####
############################
## Input:
## x: covaraites, n*p matrix. 
## a: treatments; takes values in {0, 1}## y = min{T, C}, where T is the survival time, and C is the censoring time. n*1 vector.
## delta: delta = I(T < C)I(T < tau_1), n*1 vector if y is a survival outcome.
## tau0: maximum followup time
## nu0: threshold for proportional constraint  
## testx: test data for determining the enrollment status 
## quantity: quantity of interest to optimize for survival endpoints; 
## takes value in 'survProb' for landmark
## t0: landmark time for survival probability 

## Output:
## deltax: the estimated value for the decision function in training data   
## testdeltax: the estimated value for the decision function in testing data   
## hatf: the estimated decision rule on in training data   
## testhatf: the estimated decision rule on in testing data   
## cutoff: cutoff value for the decision function

run.survival.example = function(x, a, y, r, Delta, tau0, nu0,
                                testx, quantity = 'survProb', t0){
  
  ## calculate \Delta(x)
  
  
  deltax   =  calculate_deltax(x, a, y, link = 'surv', newx = x,
               Delta = Delta, quantity = 'survProb', t0)$deltax
  deltarx  = calculate_deltax(x, a, r, link = 'bin', newx = x)$deltax
               
               
  
  ## fit cox model
  Z          = cbind(x, a, x*a)
  theta      = coxph(Surv(y, Delta) ~ Z)$coefficients
  
  ## compute lambda_0(t)
  ## lambda_l: is the basic harzard function, lambda_0(t),  at uncensored points.
  
  y_l         <- y[Delta == 1]
  Z_l         <- Z[Delta == 1, ]
  if (is.vector(Z_l)){
      Z_l = matrix(Z_l, ncol = 1)
  }
  lambda_l    <- 1/(sum.I(y_l, "<", y, exp(Z %*% theta)) + exp(Z_l %*% theta))
  
  ## compuate S_new and Lambda_new  for input y_new and Z_new
  time        =  tau0 - 10^(-5)
  S_new_1     = S_new_0 = rep(0, n)
  y_new       = time
  Z_new_1     = cbind(x, 1, x*1)
  Z_new_0     = cbind(x, 0, x*0)
  for (j in 1:n){
      Z_new_0_temp    <- Z_new_0[j,]
      Z_new_1_temp    <- Z_new_1[j,]
      
      Lambda_new      <- sum.I(y_new, ">=", y_l, lambda_l)
      
      S_new_1[j]      <- exp( - exp(Z_new_1_temp %*% theta)* Lambda_new)
      
      S_new_0[j]      <- exp( - exp(Z_new_0_temp %*% theta)* Lambda_new)
  }
  
  ## calculate effect size for different cutoffs
  sortdeltax = sort(deltax)
  delta_cuts = sortdeltax[1:ceiling(n*0.95)]
  hrs = nu_hatfcut =  probEvent_powers = prop_hatfcut = rep(0,length(delta_cuts))
  for (j in 1:length(delta_cuts)){
      cut          = delta_cuts[j]
      hatfcut      = (deltax > cut)
      nu_hatfcut[j]= mean(deltarx*(hatfcut>0))
      prop_hatfcut[j] = mean(hatfcut)
      
      S1       = mean(S_new_1[hatfcut > 0])
      S0       = mean(S_new_0[hatfcut > 0])
      lambda1  = (-log(S1))/time
      lambda0  = (-log(S0))/time
      hrs[j]    = lambda1/lambda0
      probEvent_powers[j]    = 1 - (0.5 * S1 + 0.5 * S0)
  }
  
  
  ## proposed design
  ## have a positive effect size is a minimum requirement for the proposed design
  cal_hatf   =calculate_constraint_hatf(deltax, deltarx, nu0 = nu0)
  
  hatf       = cal_hatf$hatf
  optlambda  = cal_hatf$lambda
  
  ## calculate decision functions for testdata
  testdeltax = calculate_deltax(x, a, y, link = 'surv', newx = testx, 
                                Delta = Delta, quantity = quantity, t0 = t0)$deltax
                                
  testdeltarx = calculate_deltax(x, a, r, link = 'bin', newx = testx, t0 = t0)$deltax
                                
  
  testhatf   = (testdeltax - optlambda*testdeltarx > 0)
  
  return(list(deltax = deltax, lambda = optlambda, hatf = hatf,
              testdeltax = testdeltax, testhatf = testhatf)) 
}

## generate data (example 3)

## survival endpoints
y0link <- function(n,mX,cX,A,epsilon){
    y0   <- pmin(-log(epsilon)/exp(mX + cX*A),3)
    return(y0)
}


n      =  500; 
p      =  2;
testn  =  10000;
Sigma = diag(p)
x  =  mvrnorm(n = n, mu = rep(0,p), Sigma = Sigma)
a  =  rbinom(n,1,0.5)
cY  =   (-.15 + sign(-2*x[,1]+ x[,2])*sqrt(abs(-2*x[,1] + x[,2]))/4)*1.5
mY     = 0.5 + x[,1]
epsilon    = runif(n)
survtime   = pmin(y0link(n,mY,cY,a,epsilon),50/12)
C          = runif(n,0, 50/8)
y          = pmin(survtime, C)
Delta      = as.numeric(y < C)
tau0       = max(y)

## risk outcomes binary endpoints
beta_0_risk = -.5
beta_1_risk =   1/4
gamma  = as.matrix(c(1,1))/8
delta_risk = as.matrix(c(4,-1.5))/2

expit = function(u) exp(u)/(1+exp(u))

r      =  rbinom(n,1,prob= expit(beta_0_risk + 4*a*beta_1_risk  +
x%*%gamma + 2*a*x%*%delta_risk) )


testx      =  matrix(rnorm(testn*p),ncol=p)
nu0 = 0.2

surv.result = run.survival.example(x, a, y, r, Delta, tau0, nu0,
                                testx, quantity = 'survProb', t0 = 1)
testdeltax = surv.result$testdeltax 
lambda     = surv.result$lambda