### functions required to to design max-impact design
### author: Yingqi Zhao



### function to estimate value for the contrast function, E(y|X, A = 1) - E(y|X, A = 0)

## Input:
## y: observed outcomes (could be continuous, binary or survival); if y is a survival outcome, 
## y = min{T, C}, where T is the survival time, and C is the censoring time. n*1 vector.
## a: treatments; takes values in {0, 1}
## x: covaraites, n*p matrix. 
## link: specify the endpoints type; takes value in 'con' (continuous), 'bin'(binary), 
## ‘surv' (survival)
## Delta: Delta = I(T < C)I(T < tau_1), n*1 vector if y is a survival outcome.
## quantity: quantity of interest to optimize for survival endpoints; 
## takes value in 'survProb' for landmark
## probabilty and 'RMST'for restrictive mean survival time


## Output:
## deltax: the estimated value for the contrast function
## S1: expected benefit of treatment arm
## S0: expected benefit of control arm
 
calculate_deltax = function(x, a, y, link, newx, Delta = NULL, quantity = NULL, t0 = NULL){
    if (is.vector(x)){x = matrix(x, ncol = 1)}
    if (is.vector(newx)){newx = matrix(newx, ncol = 1)}
    p  = ncol(x)
    if (link == 'bin'){
        glmMdl = glm(y ~ 1 + x + a + x:a, family = binomial)
        coeffs = coef(glmMdl)
        #coefs  = coeffs[(p+2):(2*p + 2)]
        zeromat= matrix(0,nrow = nrow(newx), ncol = ncol(newx))
        S1    = expit(cbind(1,newx,1,newx)%*%coeffs)
        S0    = expit(cbind(1,newx,0, zeromat)%*%coeffs)
        deltax = S1 - S0
    }
    
    if (link == 'con'){
        lmMdl  = lm(y ~ 1 + x + a + x:a)
        coeffs = coef(lmMdl)
        #coefs  = coeffs[(p+2):(2*p + 2)]
        zeromat= matrix(0,nrow = nrow(newx), ncol = ncol(newx))
        S1    = cbind(1,newx,1,newx)%*%coeffs
        S0    = cbind(1,newx,0,zeromat)%*%coeffs
        deltax = S1 - S0
    }
    
    if (link == 'surv'){
        zeromat= matrix(0,nrow = nrow(newx), ncol = ncol(newx))
        if  (quantity == 'survProb'){
            
            survMdl1 = cal_survival_and_density_functions(y, cbind(x, a, x*a), Delta,
            rep(t0,nrow(newx)), cbind(newx, 1, newx*1), fitting_method = "Cox")
            survMdl0 = cal_survival_and_density_functions(y, cbind(x, a, x*a), Delta,
            rep(t0,nrow(newx)), cbind(newx, 0, zeromat), fitting_method = "Cox")
            S1       = survMdl1$S
            S0       = survMdl0$S
            deltax   = S1 - S0
        }
        if (quantity == 'RMST'){
            nlesstau = sum(y <= tau0)
            ylesstau = y[y <= tau0]
            S1mat    = S0mat = matrix(0, nrow = n, ncol = nlesstau)
            for (i in 1:nlesstau){
                survMdl1 = cal_survival_and_density_functions(y, cbind(x, a, x*a), Delta,
                rep(ylesstau[i],nrow(newx)), cbind(newx, 1, newx*1),
                fitting_method = "Cox")
                survMdl0 = cal_survival_and_density_functions(y, cbind(x, a, x*a), Delta,
                rep(ylesstau[i],nrow(newx)), cbind(newx, 0, zeromat),
                fitting_method = "Cox")
                S1mat[, i]       = survMdl1$S
                S0mat[, i]       = survMdl0$S
            }
            S1       = apply(S1mat, 1, sum)
            S0       = apply(S0mat, 1, sum)
            deltax   = S1 - S0
        }
    }
    return(list(S1 = S1, S0 = S0, deltax = deltax))
}


### function to obtain the estimated decision function dictating whether
### a patient should be enrolled in the future trail

## Input:
## deltax: the estimated value for the contrast function of treatment benefit
## deltarx: the estimated value for the contrast function of risk
## nu0: the thereshold for the proportional constraint if there is one
 
## Output:
## hatf: the estimated decision function 
## lambda: the optimal lambda

calculate_constraint_hatf = function(deltax, deltarx, nu0){
    nu0prime = (nu0 - mean(deltarx*(deltax > 0 & deltarx<0)))/mean(deltarx*deltax>0)
    checkAvg = mean(deltarx[deltarx*deltax>0] * (deltax[deltarx*deltax>0] > 0))
    if (checkAvg <= nu0prime) {
        hatf   = (deltax > 0)
        optlambda = 0
    }
    if (checkAvg > nu0prime){
        fun    = function(lambda){
            mean(deltarx[deltarx*deltax>0] *(deltax[deltarx*deltax>0] -
            lambda*deltarx[deltarx*deltax>0] > 0)) - nu0prime
        }
        rootResult = uniroot(fun, c(0,5))
        optlambda = rootResult$root
        hatf      = (deltax  -  optlambda*deltarx > 0)
    }
    return(list(hatf = hatf, lambda = optlambda))
}

## functions used to calculate sample size for survival endpoint
survssfcn  = function(hr, probEvent, power = 0.8, alpha = 0.05, two.sided = T){
  if (two.sided){
    nEvents = 4*(qnorm(1-power) + qnorm(alpha/2))^2/(log(hr))^2
  }
  if (!two.sided){
    nEvents = 4*(qnorm(1-power) + qnorm(alpha))^2/(log(hr))^2
  }
  nr  = nEvents/probEvent
  return(nr)
} 

### 
## Input:
## yy is a n-dimensional vector
## yi is a N-dimensional vector
## Vi is a N-dimension vector or a N*p matrix

## Suppose FUN = "<"

## When Vi = NULL
## This program computes a n-dimensional vector, I, with
## I_j = sum_i I(yy_j < yi_i), for i = 1, ..., N, and j = 1, ..., n.

## When Vi != NULL
## It computes a n-dimensional vector or a n*p matrix, I, with
## I_{jk} = sum_i I(yy_j < yi_i)Vi_{ik}, for i= 1, ...., N, j = 1, ..., n, k = 1, ..., p.


sum.I   <- function (yy, FUN, yi, Vi = NULL) 
{
  if (FUN == "<" | FUN == ">=") {
    yy <- -yy
    yi <- -yi
  }
  
  # compute sum_yy (# (yy <  yi, i = 1,..., length(yi) ) )
  pos <- rank(c(yy, yi), ties.method = "f")[1:length(yy)] - 
    rank(yy, ties.method = "f")
  
  # compute sum_yy (# (yy >=  yi, i = 1,..., length(yi) ) )
  if (substring(FUN, 2, 2) == "=") 
    pos <- length(yi) - pos 
  
  if (!is.null(Vi)) {
    
    if (substring(FUN, 2, 2) == "=") 
      
      tmpind <- order( - yi )
    
    # compute matrix Vi, for k\th columns, for i =1, ..., nrow(Vi), sum_i( Vi_ki I(yy < y_i)  )
    
    else tmpind <- order(yi) # since yi <- -yi in the beginning, so the order is from maximum
    
    Vi <- apply(as.matrix(Vi)[tmpind, , drop = F], 2, cumsum) 
    
    return(rbind(0, Vi)[pos + 1, ]) # because pos contain 0
  }
  
  else return(pos)
  
}



## III.1, survival function estimation

## Input:
## y: min{T, C}, where T is the survival time, and C is the censoring time. n*1 vector.
## x: covaraites, n*p matrix.
## delta: delta = I(T < C)I(T < tau_1), n*1 vector.
## y_new: y for some new subjects, N*1 vector.
## x_new: covariates for some new subjects, N*1 vector.
## fitting_method: == {"Cox"}, char

## Output:
## S, survival of S(y_new | x_new). n2*1 vector. 
## Lambda, when fitting_method = "Cox", original cumulative harzard function 
## Lambda_0(y_new). n2*1 vector.
## theta, fitting parameters. p*1 vector.


cal_survival_and_density_functions    <- function(y, x, delta, y_new, x_new, 
                                                  fitting_method = "Cox"){
  
  if (fitting_method == "Cox"){
    
    ## compute parameter in cox model by coxph, and delete the intercept.
    theta       <- coxph(Surv(y, delta) ~ x)$coefficients
    
    ## compute lambda_0(t)
    ## lambda_l: is the basic harzard function, lambda_0(t),  at uncensored points.
    
    y_l         <- y[delta == 1]
    x_l         <- x[delta == 1, ]
    if (is.vector(x_l)){
      x_l = matrix(x_l, ncol = 1)
    }
    lambda_l    <- 1/(sum.I(y_l, "<", y, exp(x %*% theta)) + exp(x_l %*% theta))
    
    
    ## compuate S_new and Lambda_new  for input X_new and x_new
    
    Lambda_new      <- sum.I(y_new, ">=", y_l, lambda_l)
    
    S_new           <- exp( - exp(x_new %*% theta)* Lambda_new)
    
    
  } else {
    
    print("Wrong fitting method")
  }
  
  
  return( list( S = S_new, Lambda = Lambda_new, theta = theta) )
  
}



